"pipe" <-
function(L, D, A, G=9.8)
{
  pobj <- list(L=L, D=D, A=A, G=G)
  if (any(unlist(pobj))<=0)
    stop("Cannot have non-positive pipe parameters")
  class(pobj) <- "pipe"
  pobj
}

"initPipe" <-
function(pobj, fc, H, K, even=TRUE)
{
  if (!inherits(pobj, "pipe"))
    stop("Expecting object of class \"pipe\"")

  n <- length(fc)
  if (n%%2==0)
    stop("Expecting odd number of reaches")

  # steady state pressure by Darcy's law plus d-stream valve loss

  if (K==Inf || K<0) {
    # valve closed
    v <- 0
    h <- rep(H, n+1)
  } else {
    # valve open
    v <- sqrt((H * 2 * pobj$G) / (sum(fc)/pobj$D + K))
    vlv <- K * v^2 / (2 * pobj$G)
    h <- vlv + (H-vlv) * (1 - cumsum(c(0, fc))/sum(fc))
  }

  pobj$head <- h[seq(2-even, by=2, len=(n+1)/2)]
  pobj$flow <- rep(v, (n+1)/2)
  pobj$even <- even
  pobj$n <- n
  pobj$dt <- pobj$L/(n * pobj$A)
  pobj
}
