\name{linearPipe}

\alias{linearPipe}
\alias{jacPipe}

\title{Linearise the flow equations}

\description{The function \code{linearPipe} computes the
intercept and state transition matrix of the linear (ie
first-order) approximation to \code{stepPipe}.  The function
\code{jacPipe} is a wrapper for the C function "jacPipe" that
computes the jacobian of the fluid flow equations using a simple
numerical approximation.  Linearisation is useful for propogating
variance information.}

\synopsis{
jacPipe(pobj, fc, H, K, xleak=NA, pleak=0)
}

\usage{
linearPipe(pobj, ...)
jacPipe(pobj, ...)
}

\arguments{
  \item{pobj}{Object of class \code{pipe}}
  \item{...}{Objects passed through to \code{stepPipe},
  e.g. \code{fc}, \code{H} and \code{K}}
}

\value{A list with components \code{g} and \code{G} to be used in
a linear approximation:\deqn{\theta_t = g + G \theta_{t-1} +
\epsilon_t}{theta_t = g  + G \%*\% theta_{t-1} + epsilon_t} where
\eqn{\theta_t}{theta_t} is the state vector at time \eqn{t}, ie
the concatenation of the pressure and flow vectors at time
\eqn{t}.}

\references{West, M. and Harrison, J., 1997, Bayesian Forecasting
and Dynamic Models, 2/3ed, Springer.}

\author{Jonathan Rougier, \email{J.C.Rougier@durham.ac.uk}}

\seealso{\code{\link{stepPipe}}, which does the full non-linear
calculation.}

\examples{
pp <- pipe(L=1e4, D=0.1, A=1231)
n <- 21
dx <- pp$L / n
fc <- rep(4*0.0079*dx, n)
pp <- initPipe(pp, fc=fc, H=50, K=100)

# check approximation at steady state

lp <- linearPipe(pp, fc=fc, H=50, K=100)
nlp <- stepPipe(pp, fc=fc, H=50, K=100)
c(nlp$head, nlp$flow) - (lp$g + lp$G \%*\% c(pp$head, pp$flow))
}

\keyword{misc}
