#include <R.h>
#include "onestep.h"

static void doNothing(double x) {}

void jacPipe(double *jac, const double *cscale,
	     double *thIn, const double *fc,
	     const int *n, const int *even, const double *L,
	     const double *D, const double *A, const double *G, 
	     const double *H, const double *K, const double *xleak,
	     const double *pleak)
{
  int i, j;
  double eps0, tmp, hu, hl, *thOut1, *thOut2;

  eps0 = pow(DBL_EPSILON, 1.0/3.0);
  thOut1 = (double *) R_alloc(*n+1, sizeof(double));
  thOut2 = (double *) R_alloc(*n+1, sizeof(double));

  for (j=0; j <= *n; j++) {

    /* upper limit */

    tmp = thIn[j] + eps0 * cscale[j];
    doNothing(tmp); /* foil the optimiser! */
    hu = tmp - thIn[j];
    thIn[j] += hu;
    onestep(thOut1, thIn, fc, n, even, L, D, A, G, H, K, xleak, pleak);
    thIn[j] -= hu;

    /* lower limit */

    tmp = thIn[j] - eps0 * cscale[j];
    doNothing(tmp);
    hl = tmp - thIn[j];
    thIn[j] += hl;
    onestep(thOut2, thIn, fc, n, even, L, D, A, G, H, K, xleak, pleak);
    thIn[j] -= hl;

    /* now put them together */

    for (i=0; i <= *n; i++)
      jac[i + j*(*n+1)] = (thOut1[i] - thOut2[i]) / (hu - hl);
  }
}
